
to.plot = TRUE

library(RMySQL)
library(stats4)

drive = dbDriver("MySQL")
us    = "pppstat"
ps    = "pppstat"
db    = "sgd"
host  = "localhost"
con   = dbConnect(drive, user=us, password=ps, dbname=db, host=host)

sql = "
    SELECT FEATURE_TYPE, CHROMOSOME, pos_start, pos_end 
    FROM  `sgd_features` "

#   "WHERE  
#       `FEATURE_TYPE` = 'ORF' OR
#       `FEATURE_TYPE` = 'centromere' OR
#       `FEATURE_TYPE` = 'telomere' OR 
#       `FEATURE_TYPE` = 'transposable_element_gene' OR
#       `FEATURE_TYPE` = 'retrotransposon'
#   "

res   = dbSendQuery(con, sql)
dat   = fetch(res, n=-1)
dbClearResult(res)
dbDisconnect(con)

chr = unique(dat$CHROMOSOME)
chr = sort(as.numeric(chr))
start = as.numeric(dat$pos_start)
end   = as.numeric(dat$pos_end)
feat  = dat$FEATURE_TYPE

## 
which.orf = which(feat=="ORF" | feat=="transposable_element_gene" | feat=="retrotransposon")
y.feat = abs(start[which.orf] - end[which.orf])
## 

y.feat = abs(start - end)
y.feat = na.omit(y.feat)

y.inte = numeric(0)

# -------------------------------------------------------------------- #
# calculate the length of intergenic region
# -------------------------------------------------------------------- #
for(ch in chr){
    cstart = start[chr==ch]
    cend   = end[chr==ch]
    cmax   = max(max(cstart), max(cend))
    
    ## call is the indicator of features
    call   = numeric(cmax) 
    for(i in 1:length(cstart)){
        call[cstart[i]:cend[i]] = 1
    }
    
    ## which0 are those positions that are not features
    which0 = which(call==0)
    bound  = which(diff(which0)>1)
    
    ## start and end of intergenic regions
    inte.start = which0[bound+1]
    inte.end   = which0[bound]

    ## exclude the first and the last intergenic regions in each chrom
    inte = inte.end[-1] - inte.start[-length(inte.start)] + 1
    
    y.inte = c(y.inte, inte)
}

## find mle fit of y.feat
## gamma distribution E(X) = a*s and Var(X) = a*s^2.
## gamma.mle <- function(y, to.plot, x.max){

    y = y.feat[y.feat<10000]
    x.max = 10000
    s0 = var(y)/mean(y)
    a0 = mean(y)/s0
    
    ## log likelihood of gamma distribution
    ll = function(a=a0, s=s0)
        -sum(stats::dgamma(y, shape = a, scale=s, log=TRUE))
    
    (fit = mle(ll))
    est = coef(fit)
    
    if(to.plot){
        quartz(width=8, height=6)
        x = 1:x.max
        z = dgamma(x, shape=est[1], scale=est[2])
        d = density(y)
        y.max = max(max(d$y),max(z))
        # plot(d, xlim=c(0,x.max), )
        hist(y, ylim=c(0, y.max), freq=FALSE, main="ORF + Ty")
        lines(x, z, col="orange")
    }
    
    est
##         a          s 
##  3.151819 517.463080 
## }

    y = y.inte[y.inte<500 & y.inte>100]
    x.max=500
    x.min=100
    m = mean(y)
    s = sd(y)
    est = list(mean=m, sd=s)
    
    if(to.plot){
        quartz(width=8, height=6)
        x = 1:x.max
        z = dnorm(x, mean=m, sd=s)
        d = density(y)
        y.max = max(max(d$y),max(z))
        # plot(d, xlim=c(x.min,x.max), ylim=c(0, y.max))
        hist(y, ylim=c(0, y.max), freq=FALSE, main="intergenic region")
        lines(x, z, col="orange")
    }
    
    est

