setwd("/Users/wei/Softwares/R/ss.hmm/_R")

source("backward.R")
source("check.input.R")
source("duration.R")
source("emiss.R")
source("forward.R")
source("get.ends.R")
source("logsumexp.R")
source("post.prob.R")
source("viterbi.R")
source("plot.hmm.path.R")
source("get.end.R")

# ------------------------------------------------------------------- #
# taking log or not
# ------------------------------------------------------------------- #

log.p = TRUE

# ------------------------------------------------------------------- #
# parameters for logsumexp
# ------------------------------------------------------------------- #

cut = 0.5

# ------------------------------------------------------------------- #
# parameters for emission probability
# ------------------------------------------------------------------- #

b.limit  = 0.001
log.zero = -100.0
emissPar = c(b.limit, log.zero)

# ------------------------------------------------------------------- #
# length of each step
# ------------------------------------------------------------------- #

step = 50

# ------------------------------------------------------------------- #
# parameters for duration probability
# ------------------------------------------------------------------- #

D   = 200
lb  = 100
ub  = 500
mu3 = 500
sd3 = 300
mu4 = 500
sd4 = 300
shape = 1.68
scale = 831.8

# ------------------------------------------------------------------- #
# duration probability
# ------------------------------------------------------------------- #

du = duration(D, step, shape, scale, lb, ub, mu3, sd3, mu4, sd4, 0)
du[1:2,] = 0
du[1:10, 1] = 0
du = apply(du, 2, function(v){ v/sum(v) })
apply(du, 2, sum)

#
# write.table(du, file = "duration.txt", quote = FALSE, sep = "\t",
#             row.names = FALSE, col.names = FALSE)

# ------------------------------------------------------------------- #
# transition probability: state 1-4
# ------------------------------------------------------------------- #
#   1. flat line long (nucleosome)
#   2. flat line short (nucleosome free)
#   3. line of negative slope (followed by flat line short)
#   4. line of positive slope (followed by flat line long)

a = matrix(0, nrow=4, ncol=4)

a[1,3] = 1
a[2,4] = 1
a[3,2] = 0.4
a[3,4] = 0.6
a[4,1] = 1

# ------------------------------------------------------------------- #
# hmm initial probability: state 1-4
# ------------------------------------------------------------------- #

pi = c(0.75, 0.1, 0.1, 0.05)

# ------------------------------------------------------------------- #
# read in data, signals on chromosome 3
# ------------------------------------------------------------------- #
library(RMySQL)
drive= dbDriver("MySQL")
us   = "pppstat"
ps   = "pppstat"
db   = "histone"
host = "localhost"
con  = dbConnect(drive, user=us, password=ps, dbname=db, host=host)

sql="select * from  H3_vs_input_signal_data where chrom='chr3' order by pos"

res  = dbSendQuery(con, sql)
dat  = fetch(res, n=-1)
dbClearResult(res)
dim(dat)

# ------------------------------------------------------------------- #
# merge data into segments 
# ------------------------------------------------------------------- #

## start of each step
start.1 = 1
start.n = (ceiling(max(dat$pos)/step) - 1)*step + 1
starts  = seq(start.1, start.n, by=step)
length(starts)

m.factor = ceiling(dat$pos/step)
m.signal = tapply(dat$signal, m.factor, mean)
m.size   = tapply(dat$signal, m.factor, length)
id    = as.numeric(names(m.size))
merge = data.frame(id, n=m.size, chr=3, start=starts[id], signal=m.signal)
dim(merge)
merge[1:10,]
merge[(nrow(merge)-10):nrow(merge),]

dif = diff(merge$start)
which(dif>step) 
which(m.size<=2)

# ------------------------------------------------------------------- #
# fitting hmm model 
# ------------------------------------------------------------------- #

n1 = 288
n2 = 388
n2 = 589

dat.sub = dat[dat$pos>=merge$start[n1] & dat$pos<merge$start[n2],]
x   = dat.sub$signal
p   = dat.sub$pos

## make sure p start from a value smaller than "step"
p  = p - floor(min(p)/step)*step
D1 = ceiling(max(p)/step)

quartz(width=10, height=5)
plot(p, x, xlab="position", ylab="signal", type="l")

x = x[1:990]
p = p[1:990]

# ------------------------------------------------
# forward backward
# ------------------------------------------------

date()
x.end = get.ends(x, p, step, b.limit)
date()
f = forward(x, p, a, du, D, pi, step, x.end)
date()
b = backward(x, p, a, du, D, step, x.end)
date()
postP = post.prob(f, b, D)
date()

postS = apply(postP, 1, which.max)

postscript("postP.ps")
plot(p, x, type="l")
xS = 50*(1:D1) - 25

for(i in 1:4){
    xI = xS[postS==i]
    yI = rep(0, length(xI))
    points(xI, yI, pch=as.character(i))
}
dev.off()

# ------------------------------------------------
# viterbi
# ------------------------------------------------

hmmPath = viterbi(x, p, a, du, D, pi, step)
postscript("viterbi.ps")
plot(hmmPath)
dev.off()
